<?php

	define("CONFIGMANAGER_DEFAULT",1);
	define("CONFIGMANAGER_ACTIVE",2);

	
	class ConfigManager		{
		
		var $arrConfig = array();
		
		var $iId = -1;
		var $strName = "";
		var $bActive = FALSE;
		var $bDefault = FALSE;

		// Pattern to decode the compiled PHP-file (includes German special characters)
		var $DECODE_PATTERN = 
			"/(CONFIG\[['\"])([A-Za-z0-9_]*)(['\"]\])([\s\=]*)(['\"]{0,1})([A-Za-z0-9\/\\\@\_\s-\.\,]*)(['\"]{0,1};)/i";
		
			
		/*
			Set config var
		*/
		function SetConfigVar($strName,$strValue)	{
			$this->arrConfig[strtoupper($strName)] = $strValue;
			return TRUE;
		}

		
		function GetConfigVar($strName)		{
			if(isset($this->arrConfig[strtoupper($strName)]))	{
				return $this->arrConfig[strtoupper($strName)];
			}
			return FALSE;
		}
		
		
		function UnsetConfigVar($strName)	{
			if(isset($this->arrConfig[strtoupper($strName)]))	{
				unset($this->arrConfig[strtoupper($strName)]);
			}
			return TRUE;
		}
		

		/*
			O/R-Mapping
		*/	
		function ReadConfig($iId)	{
			
			$DB = new SQLDatabase();
			if(!$DB->dbconnect())	{	
				return FALSE;	
			}
			
			$sql = "SELECT * FROM cf_config WHERE configid=".$iId;
			$data = $DB->dbselect($sql);
			
			if(is_array($data) && sizeof($data)>0)	{
				$this->MapData($data[0]);
			} else {
				return FALSE;
			}
			return TRUE;
		}
		
		
		function ReadSpecialConfig($iType=CONFIGMANAGER_ACTIVE)		{
			$DB = new SQLDatabase();
			if(!$DB->dbconnect())	{	
				return FALSE;	
			}
			
			$sql = "SELECT * FROM cf_config ".
				"WHERE ".($iType==CONFIGMANAGER_ACTIVE?"configactive=1":"configdefault=1")." ".
				"LIMIT 1";
			$data = $DB->dbselect($sql);
			
			if(is_array($data) && sizeof($data)>0)	{
				$this->MapData($data[0]);
			} else {
				return FALSE;
			}
			return TRUE;
		}
		
		
		// Set the active configuration as default
		function SetAsDefault()	{
			
			if($this->iId <=0)	{
				return FALSE;
			}
			
			$DB = new SQLDatabase();
			if(!$DB->dbconnect())	{
				return FALSE;
			}
					
			$sql = "UPDATE cf_config SET 
				configdefault=IF(configid=".$this->iId.",1,0)";
			$iAffRows = $DB->update($sql);			
			
			return TRUE;
		}
		
		
		function SetAsActive()	{
			
			if($this->iId <=0)	{
				return FALSE;
			}
			
			$DB = new SQLDatabase();
			if(!$DB->dbconnect())	{
				return FALSE;
			}
					
			$sql = "UPDATE cf_config SET 
				configactive=IF(configid=".$this->iId.",1,0)";
			$iAffRows = $DB->dbupdate($sql);			
			
			return TRUE;
		}
		
		
		function UpdateConfig($iType=CONFIGMANAGER_ACTIVE)		{

			// Database
			$DB = new SQLDatabase();
			if(!$DB->dbconnect())	{
				return FALSE;
			}
			
			$sql = "UPDATE cf_config SET ".
				"configname='".$this->strName."', ". 
				"configeditdate=now(), ".
				"configspec='".serialize($this->arrConfig)."' ".
				"WHERE ".($iType==CONFIGMANAGER_ACTIVE?"configactive":"configdefault")."=1";
			$iAffRows = $DB->dbupdate($sql);
			
			if($iAffRows<=0)	{
				// Special configuration could not be updated
				// Create new one
				$this->InsertConfig();
				$this->SetAsActive();
			}
			return TRUE;
		}
		
		
		function InsertConfig()		{
			
			$DB = new SQLDatabase();
			if(!$DB->dbconnect())	{
				return FALSE;
			}		
			
			$sql = "INSERT INTO cf_config (	
				configactive,configdefault,configname,configeditdate,configspec) VALUES (".
				($this->bActive?"1":"0").",".($this->bDefault?"1":"0").",'".$this->strName.
				"',now(),'".serialize($this->arrConfig)."')";

			$iInsertId = $DB->dbinsert($sql);
			if($iInsertId>0)	{
				$this->iId = $iInsertId;
			} else {
				return FALSE;
			}
			return TRUE;
		}
		
		
		/*
			Save config to file
		*/
		function SaveConfigToFile($strFile,$bBackup=TRUE)		{

			// Create backup file
			if($bBackup && is_file($strFile) && is_readable($strFile))	{
				if(!copy($strFile,dirname($strFile)."/".basename($strFile,".php").".backup.php"))	{				
					return FALSE;				
				}
			}
			
			$hFile = @fopen($strFile,"w");		
			if($hFile)	{
				@fputs($hFile,$this->CompileConfig());
				@fclose($hFile);
			} else {
				return FALSE;
			}
			return TRUE;		
		}
		
		
		function CompileConfig()	{
			
			$strConfig = "<?php\r\n".
			 	"// Compiled configuration for cts ClassAd\r\n".
				"// 2 0 0 7  c t s  p r o j e c t s & s o f t w a r e\r\n".
			 	"// !!! Please do not change manually !!!\r\n\r\n";

				foreach($this->arrConfig as $key=>$element)		{
				$strConfig.= "\t\$GLOBALS['CONFIG']['".strtoupper($key)."'] = ";

				if(is_bool($element))	{
					$strConfig.= ($element?"TRUE":"FALSE");
				} elseif(is_numeric($element))	{
					$strConfig.= (string)$element;
				} else {
					$strConfig.= "\"".$element."\"";
				}
				
				$strConfig.= ";\r\n";
			}
			
			$strConfig.= "\r\n// <<<END OF CONFIGURATION>>>\r\n".
				"?>";
			return $strConfig;
		}
		

		/**
		 * Decompile existing configuration file
		 * 
		 * @version Experimental / for interal use only 
		 * @param string PHP-file containing the configuration
		 * @access public
		 * @return boolean Success
		 */
		function DecompileConfig($strConfigFile)	{
			if(!is_file($strConfigFile))	{ return FALSE; }

			$strContent = file_get_contents($strConfigFile);
			$iFound = preg_match_all($this->DECODE_PATTERN,$strContent,$arrPatterns);
			if($iFound>0)	{
				$this->arrConfig = array();
				for($i=0;$i<sizeof($arrPatterns[2]);$i++)	{
					
					// Determine booleans
					if(strtoupper($arrPatterns[6][$i])=="TRUE")	{
						$arrPatterns[6][$i] = TRUE;
					} elseif(strtoupper($arrPatterns[6][$i])=="FALSE")	{
						$arrPatterns[6][$i] = FALSE;
					}
					$this->arrConfig[$arrPatterns[2][$i]] = $arrPatterns[6][$i];
				}
			}
			return TRUE;
		}
		
		
		function DebugOut()	{
			echo "<pre>";
			print_r($this->arrConfig);
			echo "</pre>";
			return TRUE;
		}
		
		
		function MapData($element)	{
			
			// Map and decode data
			$this->iId = $element['configid'];
			$this->strName = $element['configname'];
			$this->arrConfig = unserialize($element['configspec']);
			$this->bActive = ($element['configactive']==1?TRUE:FALSE);
			$this->bDefault = ($element['configdefault']==1?TRUE:FALSE);
			
			return TRUE;
		}
	}
	
?>