<?php

	define("PH_SUCCESS",1);
	define("ERR_PH_NO_PICTURE",-10000);
	define("ERR_PH_MOVE_FAILED",-10001);
	define("ERR_PH_TOO_BIG",-10002);
	define("ERR_PH_PARTITAL",-10003);
	define("ERR_PH_NO_UPLOAD",-10004);

	
	class PictureHandler	{

		var $arrFiles = array();
		var $iMaxSize = 204800;
		
		// Directories and filehandling settings
		var $strUploadPath = "";
		var $strBigPath = "";
		var $strThumbPath = "";
		var $strThumbPrefix = "tn_";
		
		var $strFilename = "";
		
		// Image manipulation parameters
		var $iThumbWidth = 60;
		var $iThumbHeight = 60;
		var $bGdVersion2 = TRUE;
		var $iMaxWidth = 500;
		var $iMaxHeight = 500;
		var $bAllowUpsizing = FALSE;
		var $bHighQuality = FALSE;
		
		
		/*
			Constructor
		*/		
		function PictureHandler($arrFiles,$iMaxFileSize=204800)		{
			$this->arrFiles = $arrFiles;			
			$this->SetMaxSize($iMaxFileSize);
			$this->ReadConfig();
		}
		
		
		function SetMaxSize($iMaxFileSize)	{
			$this->iMaxSize = (integer)$iMaxFileSize;	
		}
		
		
		function GetFilename()	{
			return $this->strFilename;
		}
				
		
		function Process()	{
			
			require_once("inc/upload.class.php");
			
			// Upload file
			$oUpload = new Upload($this->arrFiles);
			
			// Check upload
			if(!$oUpload->IsFileUploaded())	{
				return ERR_PH_NO_UPLOAD;
			} elseif(!$oUpload->IsFileComplete())	{
				return ERR_PH_PARTITAL;
			} elseif(!in_array($oUpload->GetFileType(),array(FILE_TYPE_JPG,FILE_TYPE_PNG)))	{
				return ERR_PH_NO_PICTURE;
			} elseif(!$oUpload->IsSizeCorrect() || ($oUpload->GetSize() > $this->iMaxSize))	{
				return ERR_PH_TOO_BIG;
			} 
			
			// Get filename and move upload
			$this->strFilename = $this->GetRandomName($oUpload->GetFileName(),$oUpload->GetFileExtension());
			$strBigFilename = $this->strUploadPath.$this->strBigPath.$this->strFilename;
			$strThumbFilename = $this->strUploadPath.$this->strThumbPath.$this->strThumbPrefix.$this->strFilename;
				
			if(!$oUpload->MoveUpload($strBigFilename))	{
				return ERR_PH_MOVE_FAILED;	
			}
			
			// Do graphic manipulations
			$this->ResizeImage($strBigFilename,$strThumbFilename);

			return PH_SUCCESS;
		}
		

		/*
			Protected
		*/
		function GetRandomName($strOrigFilename,$strExtension)	{
			return md5($strOrigFilename)."_".md5(microtime()).".".$strExtension;
		}
		
		
		function ResizeImage($strFilename,$strThumbFilename)		{
			
			require_once("inc/baseclass.inc.php");
			
			$oThumb = new Thumbnail($strFilename,$this->iThumbHeight,$this->iThumbWidth); 
			$oThumb->SetVersion($this->bGdVersion2?TRUE:FALSE);
			$oThumb->SetHighQuality($this->bHighQuality);
			$oThumb->saveImage($strThumbFilename);
			
			$oThumb->SetSize($this->iMaxWidth,$this->iMaxHeight);
			$oThumb->SetUpsizing($this->bAllowUpsizing);
			$oThumb->saveImage($strFilename);
			
			return TRUE;			
		}
		
		
		function ReadConfig()	{
			
			$this->strUploadPath = GetConfig('PD_USERGRAPHICS');
			$this->iThumbHeight = GetConfig('LY_THUMBNAIL_HEIGHT');
			$this->iThumbWidth = GetConfig('LY_THUMBNAIL_WIDTH');
			$this->bGdVersion2 = GetConfig('LY_GD_VERSION_2');
			$this->iMaxWidth = GetConfig('LY_MAX_PIC_WIDTH');
			$this->iMaxHeight = GetConfig('LY_MAX_PIC_HEIGHT');
			$this->bAllowUpsizing = GetConfig('LY_ALLOW_UPSIZING');	
			$this->bHighQuality = GetConfig("LY_PIC_HIGHQUALITY");	
			
			return TRUE;
		}
				
	}

?>